@@ -17,10 +17,25 @@
  * 4. Utility functions for handling the interface between Vim and Python.
  */
 
+#include "vim.h"
+
 #include <stdio.h>
 #include <stdarg.h>
 #include <limits.h>
 
+/* Python.h defines _POSIX_THREADS itself (if needed) */
+#ifdef _POSIX_THREADS
+# undef _POSIX_THREADS
+#endif
+
+#if defined(_WIN32) && defined (HAVE_FCNTL_H)
+# undef HAVE_FCNTL_H
+#endif
+
+#ifdef _DEBUG
+# undef _DEBUG
+#endif
+
 #include <Python.h>
 #ifdef macintosh
 # include "macglue.h"
@@ -34,8 +49,6 @@
 #define file_input	257
 #define eval_input	258
 
-#include "vim.h"
-
 /******************************************************
  * Internal function prototypes.
  */
@@ -897,7 +910,10 @@ BufferNew(BUF *buf)
     BufferObject *self;
 
     if (buf->python_ref)
+    {
 	self = buf->python_ref;
+	Py_INCREF(self);
+    }
     else
     {
 	self = PyObject_NEW(BufferObject, &BufferType);
@@ -1367,7 +1383,10 @@ WindowNew(WIN *win)
     WindowObject *self;
 
     if (win->python_ref)
+    {
 	self = win->python_ref;
+	Py_INCREF(self);
+    }
     else
     {
 	self = PyObject_NEW(WindowObject, &WindowType);
