@@ -17,6 +17,11 @@
 # include <spawno.h>		/* special MSDOS swapping library */
 #endif
 
+#ifdef HAVE_TCL
+# undef EXTERN			/* redefined in tcl.h */
+# include <tcl.h>
+#endif
+
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
@@ -24,6 +29,9 @@
 static void mainerr __ARGS((int, char_u *));
 static void main_msg __ARGS((char *s));
 static void usage __ARGS((void));
+#if defined(UNIX) || defined(VMS)
+static int file_owned __ARGS((char *fname));
+#endif
 static int get_number_arg __ARGS((char_u *p, int *idx, int def));
 
 /*
@@ -49,7 +57,7 @@ mainerr(n, str)
 	"Too many \"+command\" or \"-c command\" arguments",
     };
 
-#if defined(UNIX) || defined(__EMX__)
+#if defined(UNIX) || defined(__EMX__) || defined(VMS)
     reset_signals();		/* kill us with CTRL-C here, if you like */
 #endif
 
@@ -93,7 +101,7 @@ usage()
 #endif
     };
 
-#if defined(UNIX) || defined(__EMX__)
+#if defined(UNIX) || defined(__EMX__) || defined(VMS)
     reset_signals();		/* kill us with CTRL-C here, if you like */
 #endif
 
@@ -208,6 +216,28 @@ usage()
     mch_windexit(1);
 }
 
+#if defined(UNIX) || defined(VMS)
+/*
+ * Return TRUE if we are certain the user owns the file "fname".
+ * Used for ".vimrc" and ".exrc".
+ * Use both stat() and lstat() for extra security.
+ */
+    static int
+file_owned(fname)
+    char	*fname;
+{
+    struct stat s;
+# ifdef UNIX
+    uid_t	uid = getuid();
+# else	 /* VMS */
+    uid_t	uid = ((getgid() << 16) | getuid());
+# endif
+
+    return !(mch_lstat(fname, &s) != 0 || s.st_uid != uid
+	    || mch_stat(fname, &s) != 0 || s.st_uid != uid);
+}
+#endif
+
 #if defined(GUI_DIALOG) || defined(CON_DIALOG)
 static void check_swap_exists_action __ARGS((void));
 
@@ -298,6 +328,10 @@ main
     __uname_control = __UNAME_NO_PROCESS;
 #endif
 
+#ifdef HAVE_TCL
+    Tcl_FindExecutable(argv[0]);
+#endif
+
 #ifdef MEM_PROFILE
     atexit(vim_mem_profile_dump);
 #endif
@@ -319,6 +353,10 @@ main
     gui_prepare(&argc, argv);	/* Prepare for possibly starting GUI sometime */
 #endif
 
+#if defined(HAVE_DATE_TIME) && defined(VMS) && defined(VAXC)
+    make_version();
+#endif
+
     /*
      * Allocate space for the generic buffers (needed for set_init_1() and
      * EMSG2()).
@@ -1010,17 +1048,11 @@ main
 	 */
 	if (p_exrc)
 	{
-#ifdef UNIX
-	    {
-		struct stat s;
-
-		/* if ".vimrc" file is not owned by user, set 'secure' mode */
-		if (mch_stat(VIMRC_FILE, &s) || s.st_uid != getuid())
-		    secure = p_secure;
-	    }
-#else
-	    secure = p_secure;
+#if defined(UNIX) || defined(VMS)
+	    /* if ".vimrc" file is not owned by user, set 'secure' mode */
+	    if (!file_owned(VIMRC_FILE))
 #endif
+		secure = p_secure;
 
 	    i = FAIL;
 	    if (fullpathcmp((char_u *)USR_VIMRC_FILE,
@@ -1042,11 +1074,9 @@ main
 
 	    if (i == FAIL)
 	    {
-#ifdef UNIX
-		struct stat s;
-
+#if defined(UNIX) || defined(VMS)
 		/* if ".exrc" is not owned by user set 'secure' mode */
-		if (mch_stat(EXRC_FILE, &s) || s.st_uid != getuid())
+		if (!file_owned(EXRC_FILE))
 		    secure = p_secure;
 		else
 		    secure = 0;
@@ -1443,6 +1473,11 @@ main
     mch_set_winsize_now();	    /* Allow winsize changes from now on */
 #endif
 
+    /* If ":startinsert" command used, stuff a dummy command to be able to
+     * call normal_cmd(), which will then start Insert mode. */
+    if (restart_edit)
+	stuffReadbuff((char_u *)"\034\016");
+
     /*
      * main command loop
      */
